'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _shallowequal = require('shallowequal');

var _shallowequal2 = _interopRequireDefault(_shallowequal);

var _reactRedux = require('react-redux');

var _reduxRestate = require('redux-restate');

var _reduxRestate2 = _interopRequireDefault(_reduxRestate);

var _hoistReactStatics = require('hoist-react-statics');

var _hoistReactStatics2 = _interopRequireDefault(_hoistReactStatics);

var _reactDisplayName = require('react-display-name');

var _reactDisplayName2 = _interopRequireDefault(_reactDisplayName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var getStores = function getStores(stores, props, context) {
  var result = {};
  Object.keys(stores).forEach(function (key) {
    var value = stores[key];
    if (typeof value === 'string') {
      var store = props[value] || context[value];
      if (store) {
        result[key] = store;
      } else {
        throw new Error('restate: unable to found store "' + value + '" for key "' + key + '"');
      }
    } else {
      result[key] = value;
    }
  });
  return result;
};

var nullFn = function nullFn() {
  return {};
};

var noChildren = { children: undefined };

var restate = function restate(baseStores, composeState, routeDispatch) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : nullFn;
  return function (WrappedComponent) {
    if (options && typeof options !== 'function') {
      throw new Error('react-redux-restate: options should be an option');
    }
    var basicOptions = options({});
    var storeKey = basicOptions.storeKey || 'store';
    var restateKey = basicOptions.restateKey || 'store';

    var Provider = (0, _reactRedux.createProvider)(restateKey);

    var contextTypes = _defineProperty({}, storeKey, _propTypes2.default.any);

    Object.keys(baseStores).forEach(function (key) {
      var value = baseStores[key];
      if (typeof value === 'string') {
        contextTypes[value] = _propTypes2.default.any;
      }
    });

    var ignoredProps = {};
    var deeperProps = basicOptions.deeperProps || [];

    (basicOptions.ignoreProps || []).forEach(function (prop) {
      ignoredProps[prop] = undefined;
    });

    var compareProps = function compareProps(nextProps, currentProps) {
      var additionalIgnore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      return (0, _shallowequal2.default)(_extends({}, nextProps, ignoredProps, additionalIgnore), _extends({}, currentProps, ignoredProps, additionalIgnore)) && deeperProps.reduce(function (acc, line) {
        return acc && (0, _shallowequal2.default)(nextProps[line], currentProps[line]);
      }, true);
    };

    var RestateComponent = function (_Component) {
      _inherits(RestateComponent, _Component);

      function RestateComponent(props, context) {
        _classCallCheck(this, RestateComponent);

        var _this = _possibleConstructorReturn(this, (RestateComponent.__proto__ || Object.getPrototypeOf(RestateComponent)).call(this));

        _this.composeState = function (stores) {
          return composeState(stores, _this.propsOverride || _this.props);
        };

        _this.routeDispatch = function (dispatches, event) {
          return routeDispatch(dispatches, event, _this.propsOverride || _this.props);
        };

        _this.stores = _extends({}, getStores(baseStores, props, context), {
          default: props[storeKey] || context[storeKey]
        });
        _this.propsOverride = props;
        _this.store = (0, _reduxRestate2.default)(_this.stores, _this.composeState, _this.routeDispatch, _this.getOptions());
        _this.propsOverride = null;
        return _this;
      }

      _createClass(RestateComponent, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
          this.store.initialize();
        }
      }, {
        key: 'shouldComponentUpdate',
        value: function shouldComponentUpdate(nextProps) {
          return !compareProps(nextProps, this.props);
        }
      }, {
        key: 'componentDidUpdate',
        value: function componentDidUpdate(prevPops) {
          if (!compareProps(prevPops, this.props, noChildren)) {
            this.store.replaceOptions(options(this.props));
            this.store.update();
            this.propsOverride = null;
          }
        }
      }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
          this.store.unsubscribe();
        }
      }, {
        key: 'getOptions',
        value: function getOptions() {
          return Object.assign(options(this.props), {
            noAutoSubscribe: true
          });
        }
      }, {
        key: 'render',
        value: function render() {
          return _react2.default.createElement(
            Provider,
            { store: this.store },
            _react2.default.createElement(WrappedComponent, this.props)
          );
        }
      }]);

      return RestateComponent;
    }(_react.Component);

    RestateComponent.contextTypes = contextTypes;


    (0, _hoistReactStatics2.default)(RestateComponent, WrappedComponent);
    RestateComponent.displayName = 'reconnected(' + (0, _reactDisplayName2.default)(WrappedComponent) + ')';

    return RestateComponent;
  };
};

exports.default = restate;